#  This file is "sourced", DO NOT invoke a shell here!

setenv BC_PRIVATE ""  #INSTALL
if ( ! $?BC_REPOSITORY ) then
    setenv BC_REPOSITORY ""
endif

# ------------------------------------------------------------------
#       AutoSelecting a user-defined Repository
#
#  If the env variable BC_REPOSITORY is set to an absolute path,
#  that path is set into BC_PRIVATE, in effect, selecting a specific
#  set of repository directories (BriefCase, Stage and projRCS) for
#  this BC.login session.
# 
#  To switch to another repository, set BC_REPOSITORY to the path
#  to its set of repository directories and source 
#
#    $BC_REPOSITORY/BC.login
#
#  again.
#
if ( $?BC_REPOSITORY ) then
  if ( -d "$BC_REPOSITORY" ) then 
    setenv BC_RepoPath `echo $BC_REPOSITORY| awk 'BEGIN{StdErr="cat 1>&2"} $1 ~ /^\//{print $1;exit} {print "Error - BC_REPOSITORY not abs. path"|StdErr;print "";exit}'`
    if ( "$BC_RepoPath" != "" && -d "$BC_RepoPath" ) then
	if ( -d $BC_RepoPath/BriefCase ) then
            if ( -d $BC_RepoPath/Stage ) then
		setenv BC_PRIVATE $BC_RepoPath
	    else
	        echo "Error - $BC_RepoPath is not a /BriefCase Repository"
	    endif
	else
	    echo "Error - $BC_RepoPath is not a /BriefCase Repository"
	endif
    else
	echo "Error - $BC_RepoPath is not a /BriefCase Repository"
    endif

    # If this is run on a client host, check that BC_RepoPath is valid
    # on the server
    #
    set BCSERVER=`bash $BC_PRIVATE/BriefCase/showBCSERVER`  # call bash helper script
    if ( `uname -n` != $BCSERVER ) then
        set RBCPATH=`BCrsh $BCSERVER "if [ -d $BC_RepoPath/BriefCase ];\
                        then echo OK;\
                        else echo NG;\
                        fi" >&/dev/null`
        if ( "x$RBCPATH" == xNG ) then
           echo "Error - $BCSERVER:$BC_RepoPath/BriefCase not a directory"
           echo "   Client and server paths to $BC_RepoPath/BriefCase must"
           echo "   be identical at this time.  Please work with the system"
           echo "   administrator to ensure this concurrence.  Please see"
           echo "   /BriefCase/docs/TechNotes/NO-NFShowto.txt for details."
        endif
    endif

  endif
endif

#----------------------------------------------------------------------
#
#    C-Shell version of BC.profile
#
#  /BriefCase(tm) Setup  (C) 1992,1995 Applied Computer Sciences, inc.
#
# BC_PRIVATE must be empty or point to /BriefCase's parent directory
#            for a private/test/local installed version.
#
echo "\
Welcome to /BriefCase!\
\
    Connecting to /BriefCase Server ...\
"
#  If the /Stage/home/$LOGNAME directory is not writable, /BriefCase
#  cannot migrate files between the client and server and many commands
#  will fail.
#
set ERRM=`touch $BC_PRIVATE/Stage/home/$LOGNAME.$$ `
set RC=$status
/bin/rm -f $BC_PRIVATE/Stage/home/$LOGNAME.$$
if ( $RC != 0 ) then
  echo "\
ERROR: /BriefCase connect failed: Staging area ($BC_PRIVATE/Stage/home/$LOGNAME)\
       is not writable by host: `uname -n`\
       and/or user $LOGNAME.\
\
  Notify the SysAdmin for this client host and/or the /BriefCase(tm)\
  Administrator of this error ASAP!\
 \
  DO NOT attempt to use /BriefCase commands until this problem is resolved!\
\
"
else
  # The following set ... source sequence mimics the BCclean() function
  # in BC.profile
  #  start - BCclean() 
    set TF = /tmp/bcLogin.$$
    set AWK = awk
    if ( `uname -s` == "SunOS" ) then
	set AWK = nawk
    endif
    set | grep '[Pp][Aa][Tt][Hh]' | grep BriefCase | $AWK ' \
        { \
                tc = split($0, t, "("); \
                printf "set %s = (", t[1]; \
                tca = split(t[2], a, " "); \
                sep = ""; \
                for (n = 1; n <= tca; n++){ \
                        arg = a[n]; \
                        sub(/\)$/,"",arg);\
                        if ( arg ~ /\/BriefCase/ ) continue; \
                        printf "%s%s", sep, arg; \
                        sep = " "; \
                }; \
                printf")\n"\
        }' > $TF
    #echo "###"
    #cat $TF
    #echo "###"
    source $TF
    /bin/rm -f $TF
    setenv | grep '[Pp][Aa][Tt][Hh]' | grep BriefCase | $AWK ' \
        {       Stderr = "/bin/sh -c cat 1>&2"\
                tc = split($0, t, "="); \
                printf "setenv %s ", t[1]; \
                tca = split(t[2], a, ":"); \
                sep = ""; \
                for (n = 1; n <= tca; n++){ \
                        arg = a[n]; \
                        sub(/\)$/,"",arg);\
                        if ( arg ~ /\/BriefCase/ ) continue; \
		printf"n: %d tca:%d\n",n,tca | Stderr;\
                        printf "%s%s", sep, arg; \
                        sep = ":"; \
                }; \
                printf"\n"\
        }' > $TF
    source $TF
    /bin/rm -f $TF
    unset AWK
  #  end - BCclean() 

  setenv BC_PATH $BC_PRIVATE/BriefCase
  #setenv BC_REV 3.5-Beta3b
  #  grab BC_REV from the BC.profile script
  set BR=`grep BC_REV= $BC_PATH/BC.profile|awk -F\; '{print $1}'|awk -F= '{print "setenv BC_REV " $2}'`
  $BR
  
  if ( $?MANPATH ) then
	if ( "x$MANPATH" != "x" ) then
  	     setenv MANPATH "${MANPATH}:/BriefCase/man"
	else setenv MANPATH /BriefCase/man
	endif
  else setenv MANPATH /BriefCase/man
  endif
  
  #  If /BriefCase/nawk link points to an old-style awk cmd, the "-v variable"
  #  args won;t work!  Assuming a new-style awk exists and is found by
  #  the "which nawk", the /BriefCase/nawk link should be removed.
  #
  if ( -x $BC_PATH/nawk ) then
  	set NAWK=`which nawk >& /dev/null`
  	if ( "x$NAWK" != x ) then
  	   echo " \
  WARNING: /BriefCase/nawk link hides $NAWK.\
     Inform /BriefCase(tm) Administrator ASAP!\
  "
  	endif
  	unset NAWK
  endif
  
  set o_path = ($path)
  set path = ($BC_PATH $path)
  set LOGIN = $BC_PRIVATE/BriefCase/profiles/`uname -n`
  set TF = /tmp/tf$$
  if ( -x $LOGIN ) then
  	$LOGIN csh > $TF
  else
  	$BC_PRIVATE/BriefCase/profiles/default csh > $TF
  	echo BC.login: default profile used
  endif
  source $TF
  /bin/rm $TF

  # Determine server-os and CPU type
  if ( "$BCSERVER" == `uname -n` ) then
       setenv BC_SERVER_OS $BC_OS
       setenv BC_SERVER_CPU $BC_CPU
  else 
       # WARNING: Don't split the long setenv lines with \ or Sun's csh will
       # complain about "invalid null cmd"... Linux csh likes it fine.
       setenv BC_SERVER_OS `BCrsh $BCSERVER "uname -s" | awk '{ if ($1 ~ /CYGWIN/) { print "cygwin32" } else { print $1 } exit }'`
       setenv BC_SERVER_CPU `BCrsh $BCSERVER "uname -m" `
  endif
  set path=($BC_PATH $BC_PATH/$BC_CPU/$BC_OS/bin $o_path)
  set BC_INSTAT=$BC_PATH
  if ( -L $BC_PATH ) then
      BC_INSTAT=`ls -l $BC_PATH | awk '{print $9, $10, $11}'`
  f
  echo "\
       Using /BriefCase(tm) Rev $BC_REV installed at:\
            $BC_INSTAT \
         (c)1992,2022 David Knight, and distributed \
            under the terms of the GNU GPL (ver. 2, June 1991) \
   \
           Client: `uname -n`  CPU: `uname -m`  OS: `uname -s` \
           Server: $BCSERVER  CPU: $BC_SERVER_CPU OS: $BC_SERVER_OS \
"
  if ( "$BC_PRIVATE" != "" ) then
      echo "       Current Nexus: $BC_PRIVATE\
  "
  endif
  # --------------------------------------------------------------------
  #
  #    BC_DTP_PROMO feature is enabled globally
  #    by setting env. var. BC_DTP_PROMO to "yes": 
  #    
  setenv BC_DTP_PROMO yes   #INSTALL - yes enables Dev/Test/Prod promotion
  #
  #  Dev/Test/Prod runtime promotion setup used by QA_pull and dtp_promote
  #  commands.
  #    
  #  Define DTP Directory heads
  #
  setenv BC_DTP_DEV DEVpkg    #INSTALL  Development Runtime Tree Head
  setenv BC_DTP_TEST TSTpkg    #INSTALL  QA/Test Runtime Tree Head
  setenv BC_DTP_PROD PRDpkg    #INSTALL  Production Runtime Tree Head
  
  if ( "$BC_DTP_PROMO" == "yes" ) then
    #
    #  This is a good place to test if current user if in BCadmin group
    #  or possibly in a special "Project Admin" group for this project.
    #
    #  TBD - dmk
    #
    echo Runtime Promotion features enabled
    if ( $?BC_RUNTIME ) then
       if ( "$BC_RUNTIME" != "" ) then
  	  #  Set top-level Dev/Test/Prod runtime path
  	  setenv BC_PKGPATH ${BC_RUNTIME}pkg
  	  echo "Runtime Path set to ~/$BC_PKGPATH"
       endif
       else 
  	  echo "\
    WARNING: env. var. BC_RUNTIME not set;  Runtime Promotion cmds disabled!\
    "
    endif
  else
    : # Runtime Promotion features not configured.
  endif
endif
# check to see if client/server uid numbers agree (for RCPMODE=nfs only)
checkUID  
