/*  
 *  dirt.c - prints an indented directory tree
 *       (C) 1988, 1989, 1990 Dave Knight/ACSi
 *
 *   Default configuration compiles under SunOS and 4.x BSD systems
 *   To compile on a SYstem V.3.x Unix system or MSDOS, use -DSYS5
 *   compile option of insert a line like:
 *
 *      #define SYS5
 *
 *   in this source file.
 */
#define SYS5
#define SYM_LINKS
#define NULL 0l
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#ifdef SYS5
#include <unistd.h>
#include <dirent.h>
#else
#include <access.h>
#include <sys/dir.h>
#endif

#define FALSE   0

/*
struct stat *stat();
*/

char level= 0;  /* current depth of recursion */
unsigned char shell = FALSE;
char cmnds[300];
int show_files = 0;  /* show filenames flag */
int follow_links = 0;  /* follow symbolic links flag */

#define MAXLVL 25   /* Arbitrary max for subdir. depth (recursion level) */

main (argc, argv)
int argc;
char **argv;
{
    char path[256], basepath[256], note[10] ;
    int n;
	int c;
	extern char *optarg;
	extern int optind;

#ifdef SYS5
    getcwd(basepath, sizeof(basepath));
#else
    getwd(basepath);
#endif

    while((c=getopt(argc, argv, "lf")) != EOF)
    {   switch ( c )
		{
            case 'f':
                show_files = 1;
                break;
#ifdef SYM_LINKS
            case 'l':
                follow_links = 1; /* Follow symbolic links */
                break;
#endif
            default:
                printf("Unknown option: %s\n", argv[n]);
                usage();
                exit(1);
                break;
        }
    }

	/* optind is argv subscript of next arg */

    if ( optind ==  argc )
    {       
        strcpy(path, basepath);
        strcpy(note, " == (.)");
    }
    else {
		strcpy(note,"");
        strcpy(path, argv[optind]); /* 1st arg is the starting directory */
	}
	optind++;
	

    if ( optind < argc )
    {  shell = 1;
        strcpy(cmnds, argv[optind]);
    }

    /* *************  */

    if ( !shell )
    {       printf("\nDirectory Tree for %s %s\n", path , note);
#ifdef SYM_LINKS
        printf(" ( -> indicates a symbolic link )\n\n");
#endif
    }

    note[0] = 0; /* clear note string */
    subit(path);  /* call the recursive directory print rtn. */

    if ( ! shell ) printf("\n");

}  /* end of main() */

/* 
* routine to recursively list directories
*/
subit( path)
char *path;
{
    DIR *dirp, *opendir();
    struct stat buf;
#ifdef SYS5
    struct dirent *dp, *readdir();
#else
    struct direct *dp, *readdir();
#endif

    char wd[256],msg[128];
	char prev_dir[1024];
    int n;

#ifdef SYS5
        getcwd(wd, sizeof(wd));
#else
        getwd(wd);
#endif
    if(access(path, X_OK) != 0)
    {       /* (symbolic link) not a directory */
        return(0);
    }
    if( (dirp = opendir(path)) == NULL )
    {       
        printf("\n\nCannot open directory %s\n", wd, path);
        return(-1);
    }

    if ( level > MAXLVL)
    {       
        fprintf(stderr,"Recursion level more than %d\n", MAXLVL);
        return(-1);
    }
    level++;  /* bump recursion level */

	/* save cwd so we can get back, even if we follow a symbolic link */
	strncpy(prev_dir, wd, sizeof(prev_dir));  

    chdir ( path );  /* move into this level */
#ifdef SYS5
    getcwd(wd, sizeof(wd));
#else
    getwd(wd);
#endif

    dp=readdir(dirp);  /* skip directory entry for "." */
    dp=readdir(dirp);  /* skip directory entry for ".." */

    if (shell) docmnds();

    for( dp=readdir(dirp); dp != NULL; dp=readdir(dirp) )
    {
#ifdef SYM_LINKS
        lstat(dp->d_name, &buf); /* get status for BSD file */
        if ( (buf.st_mode & S_IFLNK) == S_IFLNK )
        {
			char link_to[1024];

			memset(link_to, 0, sizeof(link_to));
			if (readlink(dp->d_name, link_to, sizeof(link_to)) < 0 )
			{	perror(dp->d_name);
			}
            else
			if ( follow_links )
            {
				int rc;
               	/* Follow Symbolic links to directories here */
               	if(!shell)
               	{       indent();
                   	printf(" %s->%s\n", dp->d_name, link_to);
               	}
               	subit( link_to );
            }
            else
            {       
                if (!shell)
                {
                    indent();
                    printf(" %s->%s\n", dp->d_name, link_to );
                }
            }
			continue;
        }
#else
        stat(dp->d_name, &buf); /* get status for SYS5 file */
#endif
        /*
         *   The following test is a bit obscure.  Note that the "mode"
         *   field is not exactly a bitfield:
         *       0020000 - means Charcter Special (device) file
         *       0040000 - means Directory   !!!
         *       0060000 - means Block Special (device) file
         *       00A0000 - means Symbolic Link
         *   ONLY a "value" of 4 indicates a directory, the 040000 bit
         *   could be a directory or a Blk Special device file.
         *   Symbolic links can be to anything except another symbolic link
         */
        n=(buf.st_mode&S_IFBLK)>>12; /* extract "value" of the 2 bits */
        if( n == 4 )
        {       /*  Here it's really a subdirectory */
            int a_rc = access ( dp->d_name, X_OK );
            if ( a_rc == 0 )
            {       if ( ! shell)
                {       indent();
                    printf(" %s\n", dp->d_name );
                }
                subit( dp->d_name );
            }
            else
            {       indent();
                printf(" %s (no access)\n", dp->d_name );
            }
        }
        else
        {       if ( ! shell && show_files )
            {       indent();
                printf(" %s\n", dp->d_name );
            }
        }
        if( strcmp("named", dp->d_name) == 0)
            printf("HIT");
    }
    closedir(dirp); /* get rid of file pointer */
    chdir (prev_dir); /* back to previou level */
#ifdef SYS5
    getcwd(wd, sizeof(wd));
#else
    getwd(wd);
#endif
    level--; /* up one */
    return(0);
}

/*
*  function to provide indentation for graphical tree
*/
indent()
{
    int i;

    if ( shell ) return(0);

    if ( level < 1 ) return (0);
    for( i = 0; i < level-1; i++)
        printf(" |    ");
    printf(" |----");
}

/*
*  routine to execute user specified command string at each dirctory level
*/

docmnds()
{
    
    char wd[128];
#ifdef SYS5
    getcwd(wd, sizeof(wd));
#else
    getwd(wd);
#endif
    printf("\n\n::: AT %s \n",wd);
    printf("::: Command(s) =  ( %s )\n\n", cmnds);
    fflush(stdout);
    system ( cmnds );
}

usage()
{       printf("dirt - Print an indented directory tree with optional filenames\n");
    printf("       or execution of a command string\n\n");
    printf("Usage:  dirt [flags] top_dir [cmnds]\n");
    printf("   where:  flags=\n");
    printf("        -f  - prints filenames in each directory\n");
#ifdef SYM_LINKS
    printf("        -l  - follows symbolic links\n");
#endif
    printf("     cmnds - is a quoted list of ; delimited cmds to be\n");
    printf("         executed once in each directory. -f is ignored\n");
    printf("         if cmnds specified.\n"); 
}

#ident "$Header: /projRCS/BCdev/src/src.dirt/dirt.c,v 1.4 1999/10/14 15:57:54 dmk%acsidev2.acsi.com Exp $"
