/*
 * ----------------------------------------------------------------------------
 *
 *               ACSi Dynamic C Dependency Generator
 *
 *       (C) 1991, 1992, 1993  by: ACSi and David M. Knight
 *
 * ----------------------------------------------------------------------------
 *
 *   A LIMITED LICENSE IS GRANTED TO SNET FOR INTERNAL USE OF THE SOURCE
 *   AND DISTRIBUTION OF COMPILED AND LINKED EXECUTABLE CODE IMBEDDED IN
 *   SNET PRODUCTS, WITHOUT ROYALTY OR OTHER PAYMENT TO ACSi.
 *
 *   ANY DISTRIBUTION OF SOURCE CODE, OBJECT FILES, LIBRARIES, ETC. IN ANY FORM
 *   THAT ALLOWS IT TO BE INCORPORATED INTO ANOTHER PROGRAM IS PROHIBITED.
 *    
 * ----------------------------------------------------------------------------
 */

/*
 * -Ipath vs. -HP/-IP Pairs Backward compatibility:
 * Define HP_IP_PAIRS to support -HP/-IP pairs for dependency output as well as
 * the newer -Ipath form where one or more include paths are defined and the
 * generated dependency is path/included_file.
 *
 * When HP_IP_PAIRS not defined, only the C compiler -Ipath form is allowed.
 */

#define HP_IP_PAIRS

#define DB foo
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#define  MAXPERLINE 3
#define  MAXPLEN 256
#define  MAXIPATHS 64
#define NULL 0

#ifdef MSDOS
char objext[] = {".obj"};  /* Extension for OBJECT files */
#else
char objext[] = {".o"};  /* Extension for Unix OBJECT files */
#endif
char lintext[] = {".ln"};  /* Extension for LINT files */
int lint = 0;
int sym_OBJ = 0;
int sym_LFILES = 0;

char Ipath[MAXIPATHS][MAXPLEN];     /* -Ipath entries */
int num_Ipaths = 0;

char target_prefix[MAXPLEN]="";
#ifdef HP_IP_PAIRS
char include_prefix[MAXPLEN]="";
char header_path[MAXPLEN]="";
#endif
char hname[MAXPLEN]="";
char pname[MAXPLEN]="";
char method[1024], *mp = 0;
char obj_name[32], *onp = 0; /* alt name for OBJ= list */
char lib_name[MAXPLEN], *lnp = 0; /* Library name for targets */
char trail_paren[2];

char ec_flag = 0;                /* ESQL/C toggle */
char ec_delims[]={" \t<\"\n;"};  /* ESQL/C (incl ';') delimiters for strtok() */
char ec_token0 = '$';            /* ESQL/C $include token[0] */
char *ec_tokenp = "exec";        /* ESQL/C exec sql include token */
char *ec_tokenUCp = "EXEC";      /* ESQL/C EXEC SQL token (upper case) */
char *ec_sqlp = "sql";           /* ESQL/C exec sql sql token */
char *ec_SQLp = "SQL";           /* ESQL/C EXEC SQL token (upper case) */

char delims[]={" \t<\"\n"};  /* delimiters for strtok() */
char *dp;

FILE *fp;
FILE *ofp = 0;  /* fp for sym_OBJ, if requested */
FILE *lfp = 0;  /* fp for sym_LFILES, if requested */

int count = 0;

char iline[1024]; /* original input line, unmarred by strtok() */

char *linep = NULL;  /* copy of line to be operated upon by strtok() */
#define DEFAULT_WA_SIZE (size_t)1024
size_t wasize = DEFAULT_WA_SIZE;

static int level=0;  /* recursion level */


#ifdef ANSI
char * hp_look(char *);
char * Ipath_look(char *);
int add_Ipath(char *);
#else
char * hp_look();
char * Ipath_look();
int add_Ipath();
#endif

#ifdef ANSI
main(int argc,char **argv)
#else
main(argc,argv)
int argc;
char **argv;
#endif
{	int n = 0;

	DB(stderr,"starting...\n");

	/* Init/Set Defaults prior to command line arg processing */

	for ( n = 0; n < MAXIPATHS; n++)
	{	Ipath[n][0] = NULL;
	}

	strncpy(obj_name,"OBJ= ", sizeof(obj_name));  /* default Object List Name */
	strncpy(method,"", sizeof(method)); 
	mp = method;

    /* Command line argument processing */

	if ( argc < 2 ) 
	{	usage();  /* exits with -1 */
	}

	n = 0;
	while( ++n < argc )
	{	
		int src_ext_len = 0;

		DB(stderr,"argv[%d] = %s\n", n, argv[n]);

        /* This loop allows src files to preceed a - option 
	     * and, thus, not be affected by it
	     */
		if( argv[n][0] == '-' )
		{	proc_opt(argv[n]);
			continue;
		}
		if( ! lfp && sym_LFILES )
		{
			if((lfp=fopen("sym_LFILES","w")) == NULL )
			{	perror(argv[n]);
				exit(-1);
			}
			fprintf(lfp,"LFILES= ");
		}
		if( ! ofp && sym_OBJ )
		{	if((ofp=fopen("sym_OBJ","w")) == NULL )
			{	perror(argv[n]);
				exit(-1);
			}
			fprintf(ofp,obj_name);
		}
		if((fp=fopen(argv[n],"r")) == NULL )
		{	perror(argv[n]);
			exit(-1);
		}

		printf("# Header dependencies for %s\n", argv[n]);

#ifdef HP_IP_PAIRS
		if(include_prefix[0] != 0 && 
		   include_prefix[strlen(include_prefix)-1] != '/') 
		{	strncat(include_prefix,"/", sizeof(include_prefix));
		}
		if(header_path[0] != 0 &&
		   header_path[strlen(header_path)-1] != '/')
		{	strncat(header_path,"/", sizeof(header_path));
		}
#endif
		src_ext_len = strlen(argv[n]) - (strrchr(argv[n], '.') - argv[n]);
		DB(stderr,"DB: src_ext_len:%d, argv[n]:%s@%08x strrchr() returns:%08x\n",
			src_ext_len, argv[n], argv[n], strrchr(argv[n], '.') );

		if( ! lnp &&
			target_prefix[0] != 0 &&
			target_prefix[strlen(target_prefix)-1] != '/')
		{	strncat(target_prefix, "/", sizeof(target_prefix));
		}

		/* 
		 *  if lib_name is specified, assume user is building targets for a library
		 *  and should look like:  target_prefix/lib_name(target_name):
		 */
		if ( ! lnp  && lib_name[0] != NULL )
		{
			strncat(target_prefix, lib_name, sizeof(target_prefix));
			lnp = lib_name;
		}

		if(lint)
		{	printf("%s%.*s%s ", target_prefix,
				strlen(argv[n]) - src_ext_len, 
				argv[n], lintext);
		}

		DB(stderr,"DB: src_ext_len:%d\n", src_ext_len);

		printf("%s%.*s%s%s : %s ", target_prefix,
			strlen(argv[n]) - src_ext_len,
			argv[n], objext, trail_paren, argv[n]);

		if ( sym_OBJ ) 
		{	fprintf(ofp,"\\\n\t%s%.*s%s%s ", target_prefix,
				strlen(argv[n]) - src_ext_len,
				argv[n], objext, trail_paren);
		}

		if ( sym_LFILES ) 
		{	fprintf(lfp,"\\\n\t%s%.*s%s ", target_prefix,
				strlen(argv[n]) - src_ext_len,
				argv[n], lintext);
		}

		count=0;
		dodeps( fp );
		if ( method[0] )
		{	printf("\n\t%s\n", method);
		}
		printf("\n");
		fclose(fp);
		pdrop();  /* drop the bag */
	}

	if( sym_OBJ) 
	{	fprintf(ofp,"\n");
		fclose(ofp);
		ofp=0;
	}

	if( sym_LFILES) 
	{	fprintf(lfp,"\n");
		fclose(lfp);
		lfp=0;
	}
	return(0);
}

/*
 *  informix_stmt() - get all lines for this ';' delimited informix
 *                    preprocessor "statement" (which may be split
 *                    over several lines).
 *
 * Theory of operation:
 *
 *   If the original input line contains a ';', assume stmt is complete and
 *   return without doing anything.  Otherwise, restore the original line,
 *   append the "continuation lines" until we have a terminating ';' and
 *   reparse the "exec" token that got us here.  Upon return,  the rest
 *   of the stmt can parsed as if it was coded on one line.
 *
 */
void informix_stmt()
{
	char cline[1024];

	if (strchr(iline, ';')) 
		return;

	strcpy(linep, iline); /* restore line work area to pre-strtok state */
	while ( 1 )
	{
		if ( fgets(cline, sizeof(cline), fp) == NULL )
		{ fprintf(stderr, "mkdep: unexpected EOF in informix stmt\n");
			exit(2);
		}
		if ( strlen(linep) + strlen(cline) > wasize + 2 ) 
		{	/* expand line work area */
			wasize += DEFAULT_WA_SIZE;
			linep = realloc(linep, wasize);
		}
		strcat(linep, cline); /* append to previous line */
		if (strchr(linep, ';'))
		{	/* got a semi-colon, were done here... */
			strtok(linep, delims);  /* restart strtok internal ptr */
			break;
		}
	}
}

/*
 *  add_Ipath(char *tp) - add path from -Ipath to array, append "/"
 */
add_Ipath( tp )
char *tp;
{
	DB(stderr,"DB: add_Ipath: tp->%s\n", tp);
	strncpy(Ipath[num_Ipaths], tp, sizeof(Ipath[0]));
	strncat(Ipath[num_Ipaths], "/", sizeof(Ipath[0]));
	DB(stderr,"DB: add_Ipath: Ipath[%d] = %s\n", num_Ipaths, Ipath[num_Ipaths]);
	num_Ipaths++;
}

/*
 *  Option Processing routine
 */
proc_opt(n)
char n[];
{	
	if( strncmp("INFORMIX", &n[1], 8) == 0 )
	{	ec_flag = 1;   /* enable Informix ESQL/C include parsing */
		return(0);
	}

	if( strncmp("TP", &n[1], 2) == 0 )
	{	strncpy(target_prefix, &n[3], sizeof(target_prefix));
		return(0);
	}

#ifdef HP_IP_PAIRS
	if( strncmp("IP$", &n[1], 3) == 0 )
	{	strncpy(include_prefix, &n[3], sizeof(include_prefix));  /* symbol as literal */
		return(0);
	}

	if( strncmp("HP", &n[1], 2) == 0 )
	{	strncpy(header_path, &n[3], sizeof(header_path));
		return(0);
	}
#endif

	if( strncmp("-I", &n[0], 2) == 0 )
	{	add_Ipath(&n[2]);
		return(0);
	}

	if( strncmp("MS", &n[1], 2) == 0 )
	{	strncpy(method, &n[3], sizeof(method));
		return(0);
	}

	if( strcmp("LINT", &n[1]) == 0 )
	{	lint = 1;
		return(0);
	}

	if( strncmp("OBJ_NAME", &n[1], 8) == 0 )
	{	if ( strlen(&n[9]) == 0 )
		{
			fprintf(stderr, "error: invalid OBJ_NAME '%s'\n", &n[9]);
			exit(1);
		}
		strncpy(obj_name, &n[9], sizeof(obj_name));
		strncat(obj_name,"= ", sizeof(obj_name));
		onp = obj_name;
		sym_OBJ = 1;
		return(0);
	}

	if( strcmp("OBJ", &n[1]) == 0 )
	{	sym_OBJ = 1;
		return(0);
	}

	if( strncmp("LIB_NAME=", &n[1], 9) == 0 )
	{	strncpy(lib_name, &n[10], sizeof(lib_name));
		strncat(lib_name, "(", sizeof(lib_name));
		strncpy(trail_paren, ")", sizeof(trail_paren));
		return(0);
	}
	else
	{
		strncpy(trail_paren, "", sizeof(trail_paren));
		lib_name[0] = NULL;
	}

	if( strncmp("LFILES", &n[1], 6) == 0 )
	{	sym_LFILES = 1;
		return(0);
	}

	usage();
}

/* 
 *  dodeps() - a resursive routine to find and list nested includes
 */

#ifdef ANSI
dodeps( FILE *fp )
#else
dodeps( fp )
FILE *fp;
#endif
{	char *token, *incpath;
	int ptype;
	FILE *subfp;

	if ( ec_flag )
		dp = ec_delims;
	else
		dp = delims;

	level++;   /*  WARNING - DON'T RETURN WITHOUT decrementing level! */
	while ( fgets(iline, sizeof(iline), fp) != NULL )
	{	
		if ( linep )
		{
			free(linep);  /* release old line */
		}
		linep = malloc(DEFAULT_WA_SIZE);  /* default size line work area */
		wasize = DEFAULT_WA_SIZE;

		strcpy(linep, iline);
		token = strtok( linep , delims );  /* find 1st token on line */
		if ( ! token ) continue;

		DB(stderr,"DB token[0]=%c ec_flag=%d\n", token[0], ec_flag);
		if( token[0] == '#' ||
			( ec_flag && token[0] == ec_token0 ) ||
			( ec_flag && strcmp(ec_tokenp, token) == 0 ) ||
			( ec_flag && strcmp(ec_tokenUCp, token) == 0 )
		  )
		{  /* this is preprocessor stmt */

			/*
			 * Handle Alternate Informix forms:
			 *
			 *    exec sql include filename;
			 *    EXEC SQL include filename;
			 *
			 * as well as multi-line versions after the fashion:
			 *
			 *    EXEC
			 *        SQL
			 *           include
			 *              filename
			 *    ;
			 */
			if ( strcmp( ec_tokenp, token ) == 0 ||
			     strcmp( ec_tokenUCp, token ) == 0
			   )
			{	/* this may be an informix EXEC stmt spanning several lines */
				informix_stmt(); /* ensure that we have the whole thing */
				token = strtok(NULL, delims);
				if ( ! token ) continue;
				if ( strcmp( ec_sqlp, token) == 0 ||
				     strcmp( ec_SQLp, token) == 0 
				   )
				{	token = strtok(NULL, delims);
					if ( ! token ) continue;
					if ( strcmp( "include", token ) != 0 )
						continue;
				}
				else
					continue;
			}

#if 0
	This code would support a split "$    include" form which seems not to be
	required, per the Informix documentation

			if( (token[0] == '#' ||
			    ec_flag && token[0] == ec_token0 ) &&
				token[1] == NULL
			   )
				token = strtok( NULL, delims); /* get next token */
#else
			/* this code allows only a split "#   include" form */
			if( token[0] == '#' &&
				token[1] == NULL
			  )
				token = strtok( NULL, delims); /* get next token */
#endif
			if ( token == NULL ) continue;

			if( ( token[0] == '#' ||
			      ec_flag && token[0] == ec_token0 ) &&
				  token[1] != NULL
			  )
				token++;  /* bump past # */

			if( strcmp( "include", token ) == 0 )
			{	int toklen;
				char *tp;

				token = strtok(NULL,delims); /* get include file name */
				if ( ! token ) continue;
				toklen = strlen(token) - 1;
				
				/*
				 *  "..." includes are checked in the CWD first and, if not
				 *  found, HdrPath is checked.  <...> includes are only
				 *  checked under HdrPath.     6/94 DMK
				 */

				strncpy(pname, token, sizeof(pname));  /* Default value */
				tp = NULL;
				if( token[toklen] != '>' )
				{	/* chk for ("...") pname in CWD */
				    if ( ec_flag && token[toklen] == ';' )
				 		pname[toklen] = NULL;
					if (pfound(pname))
						continue;
	                if ( access( pname, R_OK ) == 0 ) 
						tp = pname; /* got it */
				}
				if( !tp )
				{   if ( token[toklen] == '>' )
				 		token[toklen] = NULL;
					tp = Ipath_look(token);  /* chk for pname in Ipaths */
#ifdef HP_IP_PAIRS
					if ( !tp )
					{	tp = hp_look(token);  /* chk for pname in HdrPath */
					}
#endif
				}
				if ( tp )
					token = tp;
				else
					continue;

				DB(stderr,"DB: 2 - token=%s pname=%s\n", token, pname);

				ptype = prototype( token );  /* Is it a *proto.h ? */
				if (token)
				{	if( ! ptype || (ptype && level == 1))
					{ 
						if ( ++count >= MAXPERLINE )
						{	printf("\\\n\t"); /* Continuation */
							count = 0;
						}	
						printf("%s ", token);
					}
				}
				if ( ! ptype )
				{
					if( (subfp=fopen(pname,"r")) != NULL )
					{	dodeps( subfp ); /* recursion here */
						fclose( subfp );
					}
					else
						fprintf(stderr,"Warning: included file %s not found\n",
								pname);
				}
			}
		}
	}
	level--;
}

/*
 *  prototype(fname) - examines <fname> and returns TRUE if it ends
 *                     with "proto.h", FALSE if not
 *      method: strrchr() returns a ptr to the last occurance of the 
 *              letter 'p' in the filename or NULL if not found. 
 *              strcmp() doesn't seem to care if it is passed a NULL 
 *              as its 2nd argument.
 */
prototype(fn)
char *fn;
{	char *rc1;
	int rc2, rc3;
	rc1 = strrchr(fn, 'p');
	if (rc1)
	{	rc2 = strcmp("proto.h", rc1);
		rc3 = rc2 ? 0 : 1 ;
	}
	else
		rc3 = 0;
	return ( rc3 );
}

/*
 *   Ipath_look(char *filename) - search Ipaths for file
 *
 *      returns: 1) NULL if file already in Name list and not found in Ipaths
 *               2) ptr to resolved Ipath/Filename (Ipath[n] + filename), if
 *                  found.  Name list is managed by pfound().
 *               3) global pname[] set to expanded pathname
 */
char *Ipath_look(tp)
char *tp;
{
	int nn = 0;

	if (pfound(tp))
		return (NULL);  /* Returns null is name already in list */
	while ( nn < num_Ipaths )
	{
		strncpy(pname,Ipath[nn], sizeof(pname));
		strncat(pname, tp, sizeof(pname));
		if ( access( pname, R_OK ) == 0 ) 
		{	DB(stderr,"DB: Ipath_look: resolved %s to %s\n", tp, pname);
			return(pname); /* this is the full name to print */
		}
		nn++;
	}
	DB(stderr,"DB: Ipath_look: %s not found\n", tp);
	return(NULL); /* go on to next */
}

#ifdef HP_IP_PAIRS
/*
 *   hp_look(name_ptr) - Looks for file@name_ptr in header_path
 *
 *      returns: 1) NULL if file not found or already in list
 *               2) ptr to hname =  include_prefix + filename, if found & not
 *                  already in list.  List is managed by pfound().
 *               3) global pname[] set to expanded pathname
 */
char *hp_look(tp)
char *tp;
{
	if ( header_path[0] == NULL ) return(NULL);  /* No HdrPath */
	if ( include_prefix[0] == NULL ) return(NULL); /* Nop InclPrefix */
	strncpy(pname,header_path,sizeof(pname));
	strncat(pname,tp,sizeof(pname));
	if (pfound(pname))
		return (NULL);  /* Returns null is name already in list */
	if ( access( pname, R_OK ) == 0 ) 
	{	strncpy(hname, include_prefix, sizeof(hname));
		strncat(hname, tp, sizeof(hname));
		DB(stderr,"DB: hp_look: found %s\n", pname);
		return(hname); /* this is the full name to print */
	}
	DB(stderr,"DB: hp_look: %s not found\n", pname);
	return(NULL); /* go on to next */
}
#endif

/*
 *  pfound(pn) - accumulates a linked list of absolute pathnames
 *             already printed.
 *   input: pn is a char* to the abs. pathname to be considered
 *   output: non-zero if found,
 *           zero if not found.
 */

struct bag { struct bag *next; char p[MAXPLEN]; } *bp = NULL, *wp;

pfound(pn)
char *pn;
{	int rc = 0;
	wp = bp;
	DB(stderr,"DB: pfound: lookfor<%s>\n", pn);

	while (wp)
	{	if (strncmp( wp->p, pn, MAXPLEN) == 0)
		{ 	DB(stderr,"DB: pfound: found<%s>\n", pn);
			return(1);  /* found, get out */
		}
		if (wp->next)
			wp = wp->next;
		else
		{	DB(stderr,"DB: pfound: adding<%s>\n", pn);
			wp=wp->next=(struct bag*)calloc(1,sizeof(struct bag));/*get next*/
			strncpy(wp->p, pn, MAXPLEN-1);
			wp->next = 0;
			return(0);  /* not found, added */
		}
	}
	DB(stderr,"DB: pfound: First, adding<%s>\n", pn);
	bp=(struct bag*)calloc(1,sizeof(struct bag));/*get first*/
	bp->next = 0;
	strncpy(bp->p, pn, MAXPLEN-1);
	return(0);
}

/*
 *  pdrop() - empties the bag (frees its elements)
 */
pdrop()
{	
/* drop */
	DB(stderr,"DB: pdrop:");
	wp = bp;
	while (wp)
	{	struct bag *fp;	
		fp = wp;
		wp = wp->next;
		free(fp);
	}
	bp = 0;
}

/* 
 * usage message printer
 */
usage()
{ fprintf(stderr,"Usage: mkdep [option ...] files ...\n");
  fprintf(stderr,"   Options include:\n");
  fprintf(stderr,"       -Ipath  - where path is an include directory\n");
#ifdef HP_IP_PAIRS
  fprintf(stderr,"       -IPinclude_prefix  - literal symbol: -IP'$$(INCL)'\n");
  fprintf(stderr,"       -HPheaders_path    - directory path: -HP$(INCL)\n");
#endif
  fprintf(stderr,"       -TPtarget_prefix   - literal symbol: -TP'$$(TO)'\n");
  fprintf(stderr,"       -MSmethod_string   - e.g. -MS'@make  $(@F)'\n");
  fprintf(stderr,"       -OBJ   - Generates OBJ= symbol list\n");
  fprintf(stderr,"       -OBJ_NAMEname   - Generates name= symbol list\n");
  fprintf(stderr,"       -LIB_NAME=lib_name  - for libname(target): targets\n");
  fprintf(stderr,"       -LFILES - Generates LFILES= symbol list for lint\n");
  fprintf(stderr,"       -INFORMIX - enable ESQL/C include stmt parsing\n");
  fprintf(stderr,"       -LINT   - Generates .ln targets for lint files\n");
	exit(-1);
}
foo()
{}
